--
-- ConfigurableCapacity
--
-- Authors: Wopster
-- Description: Enables you to configure capacity
--
-- Copyright (c) Wopster, 2017
--
--[[
<capacityConfigurations>
    <capacityConfiguration name="$l10n_configuration_valueDefault" price="0">
        <fillUnit fillUnitIndex="1" fillVolumeIndex="1" capacity="8000" />
        <objectChange node="0>0|7|1|4" visibilityActive="false" />
    </capacityConfiguration>
    <capacityConfiguration name="$l10n_configuration_valueDefault" price="1000">
        <fillUnit fillUnitIndex="1" fillVolumeIndex="2" capacity="10000" />
        <objectChange node="0>0|7|1|4" visibilityActive="true" />
    </capacityConfiguration>
</capacityConfigurations>
]]

ConfigurableCapacity = {}

ConfigurableCapacity.CONFIG_NAME = "capacity"
ConfigurableCapacity.DEFAULT_INDEX = 1

function ConfigurableCapacity.prerequisitesPresent(specializations)
    if not SpecializationUtil.hasSpecialization(Fillable, specializations) or not SpecializationUtil.hasSpecialization(FillVolume, specializations) then
        print("Warning - Specialization ConfigurableCapacity needs the specializations Fillable and FillVolume.")
        return false
    end

    return true
end

function ConfigurableCapacity:preLoad(savegame)
    self.setConfigurableCapacity = ConfigurableCapacity.setConfigurableCapacity
end

function ConfigurableCapacity:load(savegame)
    if self.configurations[ConfigurableCapacity.CONFIG_NAME] ~= nil then
        self:setConfigurableCapacity(self.xmlFile, self.configurations[ConfigurableCapacity.CONFIG_NAME])
    end
end

function ConfigurableCapacity:delete()
end

function ConfigurableCapacity:mouseEvent(...)
end

function ConfigurableCapacity:keyEvent(...)
end

function ConfigurableCapacity:update(dt)
end

function ConfigurableCapacity:draw()
end

---
-- @param self
-- @param configName
--
local function getConfigByConfigId(self, configName)
    local configId = self.configurations[configName]

    if configId ~= nil then
        local item = StoreItemsUtil.storeItemsByXMLFilename[self.configFileName:lower()]
        local config = item.configurations[configName][configId]

        if config ~= nil then
            return config
        end
    end

    return nil
end

---
-- @param xmlFile
-- @param configId
--
function ConfigurableCapacity:setConfigurableCapacity(xmlFile, configId)
    local config = getConfigByConfigId(self, ConfigurableCapacity.CONFIG_NAME)

    if self.fillUnits[config.fillUnitIndex] ~= nil then
        local unit = self.fillUnits[config.fillUnitIndex]

        unit.fillVolumeIndex = config.fillVolumeIndex

        if config.capacity == nil then
            config.capacity = self:getUnitCapacity(config.fillUnitIndex)
        end

        if self.isClient then
            local fillVolume = self.fillVolumes[config.fillVolumeIndex]
            local maxPhysicalSurfaceAngle = math.rad(35)

            delete(fillVolume.volume)

            fillVolume.volume = createFillPlaneShape(fillVolume.baseNode, "fillPlane", config.capacity, fillVolume.maxDelta, fillVolume.maxSurfaceAngle, maxPhysicalSurfaceAngle, fillVolume.maxSubDivEdgeLength, fillVolume.allSidePlanes)
            setVisibility(fillVolume.volume, false)

            if fillVolume.volume ~= nil then
                link(fillVolume.baseNode, fillVolume.volume)
            end
        end

        self:setUnitCapacity(config.fillUnitIndex, config.capacity)

        if self.sprayer ~= nil then
            self.sprayer.unloadInfoIndex = config.unloadInfoIndex
        end
    end

    ObjectChangeUtil.updateObjectChanges(xmlFile, "vehicle.capacityConfigurations.capacityConfiguration", configId, self.components, self)
end

---
-- @param xmlFile
-- @param baseXMLName
-- @param baseDir
-- @param customEnvironment
-- @param isMod
-- @param configItem
--
function ConfigurableCapacity.getConfigCapacitySingleItemLoad(xmlFile, baseXMLName, baseDir, customEnvironment, isMod, configItem)
    local key = baseXMLName .. ".fillUnit"

    configItem.fillUnitIndex = Utils.getNoNil(getXMLInt(xmlFile, key .. "#fillUnitIndex"), ConfigurableCapacity.DEFAULT_INDEX)
    configItem.fillVolumeIndex = Utils.getNoNil(getXMLInt(xmlFile, key .. "#fillVolumeIndex"), ConfigurableCapacity.DEFAULT_INDEX)
    configItem.capacity = getXMLInt(xmlFile, key .. "#capacity")
    configItem.unloadInfoIndex = getXMLInt(xmlFile, key .. "#unloadInfoIndex")
end

ConfigurationUtil.registerConfigurationType(ConfigurableCapacity.CONFIG_NAME, g_i18n:getText("configuration_capacity"), nil, ConfigurableCapacity.getConfigCapacitySingleItemLoad, nil, ConfigurationUtil.SELECTOR_MULTIOPTION)